unit Film;
// Auteur: Th.M. Hupkens
// Gebruik voor eigen risico.
// Let op: In een programma dat deze component gebruikt worden
// alleen de namen van de bestanden opgeslagen, NIET de plaatjes
// of de geluiden zelf. De gebruiker moet de beschikking
// hebben over de bestanden en het programma moet deze kunnen vinden.
// Als u zo'n programma verspreid dan kunt u het best alle benodigde
// bestanden (laten) installeren in dezelfde directory als het programma. 

interface uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DsgnIntf, extdlgs, extctrls, MMSystem;

type
  // Hieronder staat een "property editor". Deze maakt het inlezen van
  // de geluidsnaam via de Object Inspector makkelijker voor de gebuiker
  TGeluidProperty = class(TStringProperty)
    function GetAttributes: TPropertyAttributes; override;
    procedure Edit; override;
  end;

  // Hieronder staat een "component editor". Hiermee kunnen items worden
  // toegevoegd aan het snelmenu dat je krijgt als je (in de ontwerpfase) met
  // de rechter muisknop op de afbeelding van de component klikt.
  // Bij dubbelklikken hierop wordt het bovenste item uitgevoerd.
  TFilmEditor = class(TComponentEditor)
    function GetVerbCount: integer; override;
    function GetVerb(Index: integer): string; override;
    procedure ExecuteVerb(Index: integer); override;
  end;

  TWeergave = (WareGrootte, Zoom);
  TFilmType = (LusFilm, Normaal, EenKeerHeenEnWeer, LusHeenEnWeer);
  TFoutActie = (EvenGeenFilm, RaiseException);
  TStartFilm = (Via_Programma, StartMeteen, MuisKlik);
  TFilm = class(TGraphicControl)
  private
    Bitmap: TBitmap;
    FStartFrame, FStopFrame, FVoorFrame, FNaFrame, FTijdPerFrame,
                 FrameNr, BeginFrame, EindFrame: integer;
    Timer: TTimer;
    FWeergave: TWeergave;
    FLijst: TStringList;
    FFoutActie: TFoutActie;
    FFilmType, TempFilmType: TFilmType;
    FPen: TPen;
    FAchterBrush: TBrush;
    FAlfa, FTransparent, Bezig: boolean;
    FGeluid: AnsiString;
    FStartFilm: TStartFilm;
    FDefaultPad: AnsiString;
    Stoppen: boolean; // Geeft aan dat de film op het punt staat te gaan stoppen
    procedure SetAlfa(const Value: boolean);
    procedure SetWeergave(const Value: TWeergave);
    procedure SetAchterBrush(const Value: TBrush);
    procedure SetPen(const Value: TPen);
    procedure SetTijdPerFrame(const Value: integer);
  protected
    procedure LaadFrames(Nieuw: boolean);
    procedure Loaded; override;
    procedure NieuwFrame(Sender: TObject);
    procedure Paint; override;
    procedure SetNames(Value: TStringList);
    procedure SetFrame(const Index, Value: integer);
    procedure StyleChanged(Sender: TObject);
  public
    function FilmLoopt: boolean;
    procedure Click; override;
    procedure ToonFrame(Nr: integer); // Als Nr < 0 dan alleen achtergrond
    procedure ZetCapaciteit(Sender: TObject);
    procedure StartFilm;
    procedure StopFilm;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property DefaultPadNaam: AnsiString read FDefaultPad write FDefaultPad;
    property Achtergrond:   TBrush read FAchterBrush  write SetAchterBrush;
    property AlfabetischeLijst: boolean read FAlfa    write SetAlfa;
    property FilmType:   TFilmType read FFilmType     write FFilmType;
    property Geluid:    AnsiString read FGeluid       write FGeluid;
    property Pen:             TPen read FPen          write SetPen;
    property Weergave:   TWeergave read FWeergave     write SetWeergave;
    property FoutActie: TFoutActie read FFoutActie    write FFoutActie
             default EvenGeenFilm;
    property Frames:   TStringList read FLijst        write SetNames;
    property TijdPerFrame: integer read FTijdPerFrame write SetTijdPerFrame
             default 100;  // De tijd is in millisekonden
    property Starten: TStartFilm read FStartFilm write FStartFilm;
 // Door de volgende twee properties kan de film in gedeelten worden vertoond:
    property StartFrame: integer index 0 read FStartFrame write SetFrame
             default 0;   // eerste frame dat getoond wordt
    property StopFrame:  integer index 1 read FStopFrame  write SetFrame
             default -1;  // laatste frame dat getoond wordt
    property VoorFrame:  integer index 2 read FVoorFrame  write SetFrame
             default -1;  // Frame dat zichtbaar is voor de Film gestart is
    property NaFrame:    integer index 3 read FNaFrame    write SetFrame
             default -1;  // Frame dat zichtbaar is als de Film afgelopen is
    property Transparent: boolean read FTransparent write FTransparent default false;
    property Visible;
    property OnClick;      //
    property OnDblClick;   //
    property OnDragDrop;   //
    property OnDragOver;   //  Stel alle muis-events beschikbaar
    property OnEndDrag;    //
    property OnMouseDown;  //
    property OnMouseMove;  //
    property OnMouseUp;    //
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('LevendeVormen', [TFilm]);
  RegisterPropertyEditor(TypeInfo(AnsiString), TFilm, 'Geluid', TGeluidProperty);
  RegisterComponentEditor(TFilm, TFilmEditor)
end;

{ TFilm }

constructor TFilm.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width := 128;
  Height := 128;
  FLijst := TStringList.Create;
  Bitmap := TBitmap.Create;
  FStopFrame := -1; // Tot aan laatste plaatje
  FVoorFrame := -1; //
  FNaFrame   := -1; // -1 wil zeggen: geen zichtbaar plaatje, wel achtergrond
  FrameNr    := -1; //
  FTijdPerFrame := 100;
  Timer := TTimer.Create(Self);
  With Timer do Begin
       Enabled := false; // Zet timer meteen af!
       Interval := FTijdPerFrame;
       OnTimer := NieuwFrame End;
  FPen := TPen.Create;
  FPen.OnChange := StyleChanged;
  FAchterBrush := TBrush.Create;
  FAchterBrush.Style := bsClear;
  FAchterBrush.OnChange := StyleChanged;
  FLijst.OnChange := ZetCapaciteit
end;

destructor TFilm.Destroy;
begin
  FLijst.Free;
  Bitmap.Free;
  FPen.Free;
  FAchterBrush.Free;
  inherited Destroy
end;

procedure TFilm.LaadFrames;
var Frames: TOpenPictureDialog;
    i: integer;
begin
  Frames := TOpenPictureDialog.Create(nil);
  try
    With Frames do Begin
         Title := 'Kies een serie beelden uit de lijst';
         Filter := 'Windows Bitmaps|*.bmp';
         DefaultExt := 'bmp';
         Options := [ofHideReadOnly, ofShowHelp, ofAllowMultiSelect,
                     ofExtensionDifferent, ofPathMustExist, ofFileMustExist,
                     ofShareAware, 
              (* {Delphi4:} ofEnableSizing, {resizing mogelijk} *) ofShowHelp];
         If Execute then Begin
            If Nieuw then Begin If FVoorFRame > 0 then FVoorFrame := 0;
                                   FLijst.Clear End;
            For i := 0 to Files.Count - 1 do FLijst.Append(Files[i]);
            If FStartFrame >= FLijst.Count then FStartFrame := FLijst.Count - 1;
            Paint End
         End
  finally
    Frames.Free
  end
end;

procedure TFilm.NieuwFrame(Sender: TObject);
var Temp: integer;
begin
  If FLijst.Count = 0 then Exit;
  If Stoppen and (Sender = Timer) then Begin StopFilm; Exit End;
  If EindFrame < BeginFrame then       // <-- Film loopt terug
     Begin If FrameNr > 0 then Dec(FrameNr) else StopFilm End
  else If EindFrame = BeginFrame then FrameNr := EindFrame
  else Begin Inc(FrameNr); If FrameNr > EindFrame then FrameNr := 0 End;
  Repaint; // Forceert herstellen achtergrond
  If FrameNr = EindFrame then
     Case TempFilmType of
     Normaal: Stoppen := true; // Stopt film NADAT TijdPerFrame verstreken is
     LusHeenEnWeer, EenKeerHeenEnWeer:
     Begin Temp := EindFrame; EindFrame := BeginFrame; BeginFrame := Temp;
     If TempFilmType = EenKeerHeenEnWeer then TempFilmType := Normaal End
     End
end;

procedure TFilm.Paint;
begin
  inherited Paint;
  If csDesigning in ComponentState then Begin
     Canvas.Pen.Color := clWhite xor (Parent.Brush.Color and $FFFFFF);
     Canvas.Brush.Style := bsClear; Canvas.Pen.Style := psDash;
     Canvas.Rectangle(0, 0, Width, Height) End;
  If FLijst.Count = 0 then ToonFrame(-2)
  else ToonFrame(FrameNr)
end;

procedure TFilm.SetNames(Value: TStringList);
begin
  FLijst.Duplicates := dupAccept;
  FLijst.Assign(Value)
end;

procedure TFilm.SetTijdPerFrame(const Value: integer);
begin
  If Value = FTijdPerFrame then Exit;
  FTijdPerFrame := Value;
  Timer.Interval := Value
end;

procedure TFilm.ToonFrame(Nr: integer);
var FileNaam, Naam, Pad: AnsiString;
    FilmLoopt: boolean;
begin
  FrameNr := Nr;
  With Canvas do If Weergave <> Zoom then Begin
       Brush := FAchterBrush; Pen := FPen;
       Rectangle(0, 0, Width, Height) End; // Maak achtergrond vast in orde
  If (Nr < 0) or (Nr >= FLijst.Count) then Exit;
  If Bezig then Exit; // Voorkom dat een frame getekend wordt terwijl een fout
  Bezig := true;      // nog niet afgehandeld is
  try
    If Nr < FLijst.Count then Begin
// Zoek paden af in volgorde Lijst, Pad van Exe, DefaultDir en Huidige dir
       FileNaam := FLijst[Nr];
       If not FileExists(FileNaam) then Begin
          Naam := ExtractFileName(FLijst[Nr]);
          Pad := ExtractFilePath(Application.ExeName);
          If Pad[Length(Pad)] <> '\' then pad := Pad + '\';
          FileNaam := Pad + Naam;
          If not FileExists(FileNaam) then Begin
             FileNaam := FDefaultPad + '\' + Naam;
             If not FileExists(FileNaam) then FileNaam := Naam End
          End;
       Bitmap.LoadFromFile(FileNaam); // Nu pas laden, niet goed dan naar except
       Bitmap.Transparent := FTransparent;
       If FWeergave = Zoom then Canvas.StretchDraw(ClientRect, Bitmap)
       else Canvas.Draw
          ((Width - Bitmap.Width) div 2, (Height - Bitmap.Height) div 2, Bitmap)
       End
  except
    FilmLoopt := Timer.Enabled;
    Timer.Enabled := false; // Anders krijg je teveel foutmeldingen
    If csDesigning in ComponentState then Begin
       If not FilmLoopt then Begin // Eenvoudige foutmelding als film niet loopt
          Canvas.TextOut(10, 10, 'Bestandsfout');
          Canvas.TextOut(10, 30, FileNaam);
          Bezig := false; Exit End; 
       Case MessageDLG('Het bestand ' + FileNaam + ' kan niet geladen worden.'
         + #13 + 'Moet dit bestand verwijderd worden uit de lijst?'
                , mtError, [mbYes, mbIgnore, mbAbort], 0) of
       mrYes: With FLijst do Begin
                   Delete(Nr);
                   If FStartFrame >= Count then FStartFrame := Count - 1;
                   If EindFrame > BeginFrame then Dec(EindFrame)
                   else Dec(BeginFrame);
                   Timer.Enabled := FilmLoopt End;
       mrIgnore: Timer.Enabled := FilmLoopt;
       mrAbort: If FrameNr = VoorFrame then FrameNr := -1 else FrameNr := VoorFrame
       End
       End
    else If FFoutActie = RaiseException then raise
         else Timer.Enabled := FilmLoopt
  End;
  Bezig := false
end;

procedure TFilm.SetFrame(const Index, Value: integer);
var Val: integer;
begin
  If Value >= FLijst.Count then Val := FLijst.Count - 1 else Val := Value;
  Case Index of
  0: If Val < 0 then FStartFrame := 0 else FStartFrame := Val;
  1: FStopFrame := Val;
  2: Begin
       FVoorFrame := Val; FrameNr := Val;
       Invalidate End
  else FNaFrame := Val End
end;

procedure TFilm.SetWeergave(const Value: TWeergave);
begin
  If FWeergave = Value then Exit;
  FWeergave := Value;
  Invalidate
end;

procedure TFilm.SetAchterBrush(const Value: TBrush);
begin
  FAchterBrush := Value
end;

procedure TFilm.SetPen(const Value: TPen);
begin
  FPen := Value
end;

procedure TFilm.StyleChanged(Sender: TObject);
begin
  Invalidate
end;

procedure TFilm.StartFilm;
begin
  EindFrame := FStopFrame;
  If (EindFrame < 0) or (EindFrame >= FLijst.Count)
  then EindFrame := FLijst.Count - 1;
  BeginFrame := FStartFrame;
  FrameNr := BeginFrame;
  TempFilmType := FFilmType;
  Timer.Enabled := true;
  Invalidate; // Teken eerst het startframe
  If FGeluid <> '' then Begin
     PlaySound(nil, 0, SND_ASync); // Stop geluid als dat er is...
     PlaySound(PChar(FGeluid), 0, SND_ASync) End
end;

procedure TFilm.StopFilm;
begin
  Timer.Enabled := false;
  Stoppen := false;
  If (csDesigning in ComponentState) then FrameNr := FVoorFrame
  else If FrameNr > -1 then FrameNr := FNaFrame;
  Invalidate
end;

procedure TFilm.SetAlfa(const Value: boolean);
begin
  FAlfa := Value;
  FLijst.Sorted := FAlfa;
  Invalidate
end;

procedure TFilm.ZetCapaciteit(Sender: TObject);
begin
  FLijst.Capacity := FLijst.Count // Nodig wegens AlfabetischeLijst
end;

procedure TFilm.Loaded;
begin
  inherited Loaded;
  If csDesigning in ComponentState then Exit;
  If FStartFilm = StartMeteen then StartFilm
end;

procedure TFilm.Click;
begin
  inherited Click; // Zo kan de gebruiker Click nog ergens anders voor gebruiken
  If FStartFilm = MuisKlik then If Timer.Enabled then StopFilm else StartFilm
end;

function TFilm.FilmLoopt: boolean;
begin
  Result := Timer.Enabled
end;

{ TGeluidProperty }

procedure TGeluidProperty.Edit;
var GeluidsDialoog: TOpenDialog;
const EersteKeer: boolean = true; // Geldig voor alle TFilm objecten samen!
Begin
  GeluidsDialoog := TOpenDialog.Create(Application);
  try
    With GeluidsDialoog do Begin
         DefaultExt := 'wav'; Filter := 'Geluidsbestanden|*.wav';
         If EersteKeer then InitialDir := 'c:\Windows\Media';        // Zet hier
         EersteKeer := false;                // desgewenst een ander default pad
         Title := 'Kies uit deze lijst een geluid bij de film';
         Options := [ofHideReadOnly];
         If Execute then SetValue(FileName) End
  finally
    GeluidsDialoog.Free
  end
End;

function TGeluidProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog]
end;

{ TFilmEditor }

function TFilmEditor.GetVerbCount: integer;
begin
  If TFilm(Component).FLijst.Count = 0 then Result := 1
  else Result := 5
end;

function TFilmEditor.GetVerb(Index: integer): string;
begin
  Case Index of
  0: Result := 'Voeg plaatjes toe...';
  1: Result := 'Vervang alle plaatjes...';
  2: If TFilm(Component).FilmLoopt then Result := 'Stop test'
     else Result := 'Test &film';
  3: Result := 'Toon volgend frame';
  else Result := 'Toon vorig frame' end
end;

procedure TFilmEditor.ExecuteVerb(Index: integer);
begin
  With TFilm(Component), FLijst do Begin
    Case Index of                                    // De gebruiker koos:
    0: LaadFrames(false);                            // "Voeg plaatjes toe..."
    1: LaadFrames(true);                             // "Laad alle plaatjes..."
    2: If Timer.Enabled then StopFilm                // "Stop test"
       else StartFilm;                               // "Test film"
    3: Begin                                         // "Toon volgend frame"
         Inc(FrameNr); If FrameNr >= Count then FrameNr := 0;
         Invalidate End;
    4: Begin                                         // "Toon vorig frame"
         If FrameNr > 0 then Dec(FrameNr) else FrameNr := Count - 1;
         Invalidate End
    End
  End
end;

end.
