unit GolfTekst;
// Auteur: Th.M. Hupkens
interface uses
  ExtCtrls, Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls;

const NKleur = 5; // Het aantal kleuren kan naar believen worden uitgebreid.
                  // Breid daartoe de propertylijst van TKleuren uit met Kleur6,
                  // Kleur7, enz. en pas NKleur aan. De rest gaat vanzelf goed
type
  TFijnheid = 1..10; // Verdeelt fase over de letters, 1 = normaal

  TKleuren = class(TPersistent) // Verplicht afgeleid van TPersistent
    private
      FKleur: array[1..NKleur] of TColor;
      function GetKleur(const Index: Integer): TColor;
      procedure SetKleur(const Index: Integer; const Value: TColor);
    published
      property Kleur1: TColor index 1 read GetKleur write SetKleur default 0;
      property Kleur2: TColor index 2 read GetKleur write SetKleur default 0;
      property Kleur3: TColor index 3 read GetKleur write SetKleur default 0;
      property Kleur4: TColor index 4 read GetKleur write SetKleur default 0;
      property Kleur5: TColor index 5 read GetKleur write SetKleur default 0;
    End;

  TGolfTekst = class(TGraphicControl)
    private
      Timer: TTimer;
      FTijdPerStap, FGolfLengte, FRuimte: integer;
      FAantalKleuren, FAmplitude: word;
      FAutosize, Eentop: boolean;
      FKleuren: TKleuren;
      FFijnheid: TFijnheid;
      FAlignment: TAlignment;
      function GetTransparent:    boolean;
      function GetGolfLengte:     integer;
      function SetAantalKleuren:  word;
      function SetKleuren:        TKleuren;
      procedure SetTijdPerStap    (const Value: integer);
      procedure Tik               (Sender: TObject);
      procedure SetCaption        (const Value: TCaption);
      procedure SetTransparent    (const Value: boolean);
      procedure SetAmplitude      (const Value: word);
      procedure SetAutosize       (const Value: boolean);
      procedure SetGolfLengte     (const Value: integer);
      procedure SetAlignment      (const Value: TAlignment);
    procedure SetRuimte(const Value: integer);
    public
      Fase: double;             // Mag door de gebruiker veranderd worden
      function Actief: boolean; // TRUE als de tekst golft
      procedure Hide;           // Verberg GolfTekst en zet timer stil
      procedure Paint; override;
  // In Delphi 4 kunt u in onderstaande regel de akkolades weglaten, zodat
  // Show zonder parameters te gebruiken is
      procedure Show(const Golf: boolean{ = false});
      procedure Start;          // Start de timer, de tekst gaat golvem
      procedure Stop;           // Stop de timer, de tekst golft niet meer
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
    published
      property Alignment: TAlignment read FAlignment write SetAlignment;
      property Fijnheid: TFijnheid
               read FFijnheid write FFijnheid default 1;
      property GolfLengte: integer
               read GetGolfLengte write SetGolfLengte default 0;
      property RuimteTussenLetters: integer
               read FRuimte write SetRuimte default 0;
      property TijdPerStap: integer  // Als deze waarde 0 is doet de timer niets
               read FTijdPerStap write SetTijdPerStap default 100;
      property Transparent: boolean
               read GetTransparent write SetTransparent default true;
      property AmplitudePercentage: word
               read FAmplitude write SetAmplitude default 50;
      property Font;
      property AutoSize: boolean read FAutoSize write SetAutosize default true;
      property Caption write SetCaption;
      property Color;
      property AantalKleuren: word
               read SetAantalKleuren write FAantalKleuren default 0;
      property Kleuren: TKleuren read SetKleuren write FKleuren;
      property OnClick; // Maak alle event properties zichtbaar die u nodig heeft
      property OnDblClick; // zoals deze twee
    end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('LevendeVormen', [TGolfTekst])
end;

{ TGolfTekst }

constructor TGolfTekst.Create(AOwner: TComponent);
begin
  inherited;
  FKleuren := TKleuren.Create;
  Timer := TTimer.Create(Self);
// Als je wil dat de tekst golft tijdens ontwerp, haal dan { en } weg:
{  If not (csDesigning in ComponentState) then }
    Timer.Enabled := false;
  FTijdPerStap := 100;
  Timer.Interval := FTijdPerStap;
  Timer.OnTimer := Tik;
  FAmplitude := 50;
  FAutoSize := true;
  FFijnheid := 1;
  Width := 17; Height := 80;
  EenTop := true
end;

destructor TGolfTekst.Destroy;
begin
  FKleuren.Free;
  Timer.Free;
  inherited
end;

function TGolfTekst.GetGolfLengte: integer;
begin
  If EenTop then Result := 0 else Result := FGolfLengte
end;

function TGolfTekst.GetTransparent: boolean;
begin
  Result := not (csOpaque in ControlStyle)
end;

function TGolfTekst.Actief: boolean;
begin
  Result := Timer.Enabled
end;

procedure TGolfTekst.Hide;
begin
  Timer.Enabled := false;
  inherited
end;

procedure TGolfTekst.Paint;
var i, DeltaX, A, K, TekstBreedte: integer;
    Frequentie: double;
    TM: TTextMetricA;   // In Delphi 4 mag ook: TextMetric

const GradToRad = 2 * Pi;

begin
  inherited;
  K := FAantalKleuren;
  If K > NKleur then K := NKleur;
  With Canvas do Begin
    if not Transparent then begin
      Brush.Color := Self.Color;
      Brush.Style := bsSolid;
      FillRect(ClientRect) end
    else Brush.Style := bsClear;
    Font.Assign(Self.Font);
    TekstBreedte := Canvas.TextWidth(Caption) + Length(Caption) * FRuimte;
    DeltaX := 0; // Moet voor If FAutoSize ...
    GetTextMetrics(Handle, TM); // Font.Height is niet goed bruikbaar
    A := round(0.01 * FAmplitude *  TM.tmHeight);
    If FAutoSize then Begin
       Width := 1 + TekstBreedte;
       Height := 2 + 2 * A + TM.tmHeight End
    else
       Case FAlignment of
       taCenter: DeltaX := (Width - TekstBreedte) div 2;
       taRightJustify: DeltaX := Width - TekstBreedte - 1
       End;
    If Caption = '' then Exit;
    If FGolfLengte = 0 then Frequentie := GradToRad / Length(Caption)
    else Frequentie := GradToRad / FGolflengte;
    For i := 1 to Length(Caption) do Begin
        If FAantalKleuren > 0 then
           Font.Color := Fkleuren.Fkleur[1 + (i - 1) mod K]; // Zo krijgt de 1e letter kleur 1
        TextOut(DeltaX,
                (Height - TM.tmHeight) div 2
                - round(A * sin(Frequentie * i + Fase)),
                Caption[i]);
        Inc(DeltaX, TextWidth(Caption[i]) + FRuimte) End
    End
end;

procedure TGolfTekst.SetAmplitude(const Value: word);
begin
  If Value = FAmplitude then Exit;
  FAmplitude := Value;
  invalidate
end;

procedure TGolfTekst.SetAutosize(const Value: boolean);
begin
  If Value = FAutosize then Exit;
  FAutoSize := Value;
  If FAutoSize then invalidate
end;

procedure TGolfTekst.SetCaption(const Value: TCaption);
begin
  inherited Caption := Value;
  if EenTop then FGolfLengte := Length(Value);
  invalidate
end;

procedure TGolfTekst.SetGolfLengte(const Value: integer);
begin
  if Value = FGolfLengte then Exit;
  FGolfLengte := Value;
  EenTop := Value = 0;
  if EenTop then FGolfLengte := Length(Caption);
  invalidate
end;

procedure TGolfTekst.SetTijdPerStap(const Value: integer);
begin
  FTijdPerStap := Value;
  Timer.Interval := Value
end;

procedure TGolfTekst.SetTransparent(const Value: boolean);
begin
  if Value = Transparent then Exit;
  if Value then ControlStyle := ControlStyle - [csOpaque]
  else ControlStyle := ControlStyle + [csOpaque];
  invalidate
end;

procedure TGolfTekst.Show;
begin
  inherited Show;
  Timer.Enabled := Golf
end;

procedure TGolfTekst.Start;
begin
  Timer.Enabled := true
end;

procedure TGolfTekst.Stop;
begin
  Timer.Enabled := false
end;

procedure TGolfTekst.Tik(Sender: TObject);
begin
  Fase := Fase + 1 / FFijnheid; If Fase > 2 * Pi then Fase := 0.0;
  invalidate
end;

function TGolfTekst.SetAantalKleuren: word;
begin
  Result := FAantalKleuren;
  invalidate
end;

function TGolfTekst.SetKleuren: TKleuren;
begin
  Result := FKleuren;
  if FAantalKleuren > 0 then invalidate
end;

procedure TGolfTekst.SetAlignment(const Value: TAlignment);
begin
  FAlignment := Value;
  invalidate
end;

procedure TGolfTekst.SetRuimte(const Value: integer);
begin
  If Value = FRuimte then Exit;
  FRuimte := Value;
  invalidate
end;

{ TKleuren }

function TKleuren.GetKleur(const Index: integer): TColor;
begin
  Result := FKleur[Index]
end;

procedure TKleuren.SetKleur(const Index: Integer; const Value: TColor);
begin
  FKleur[Index] := Value
end;

end.
