unit VormKlok;
// Auteur: Th.M. Hupkens
{$R-}
interface

uses
  SysUtils, Classes, Graphics, Controls, Vormen, Extctrls;

type
  TPercentage = 0..100;
  TWijzerSoort = (W_Uur, W_Min, W_Sec);
  TCijfers = (Geen, DrieZesNegenTwaalf, Alle);
  TWijzer = class(TPersistent)
    private
      FLengte: TPercentage;
      FDikte: integer;
      FKleur: TColor;
    published
      property Lengte: TPercentage read FLengte write FLengte;
      property Dikte: integer read FDikte write FDikte default 1;
      property Kleur: TColor read FKleur write FKleur;
    End;
  TVormKlok = class(TVorm)
    private
      Klok: TTimer;
      FDigital, FSeconde: boolean;
      FWijzers: array[TWijzerSoort] of TWijzer;
      Fx_c, Fy_c, Oud_Min: integer;
      FFactorCijferRing: word;
      FGetallen: TCijfers;
      CijferSet: set of 1..12;
    protected
      procedure NieuweStand(Sender: TObject);
      function  GetWijzer(index: integer): TWijzer;
      procedure SetWijzer(index: integer; Value: TWijzer);
      function  GetGetallen: TCijfers;
      procedure SetGetallen(Value: TCijfers);
      procedure SetCijferRing(Value: word);
      function  GetCijferRing: word;
      procedure Paint; override;
    public
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
    published
      property Cijfers: TCijfers read GetGetallen write SetGetallen default Alle;
      property Font;
      property StraalCijfer_Ring: word read GetCijferRing write SetCijferRing default 100;
      property Digital: boolean read FDigital write FDigital default false;
      property Seconde: boolean read FSeconde write FSeconde default false;
      property WijzerUur: TWijzer index Ord(W_Uur) read GetWijzer write SetWijzer;
      property WijzerMinuut: TWijzer index Ord(W_Min) read GetWijzer write SetWijzer;
      property WijzerSeconde: TWijzer index Ord(W_Sec) read GetWijzer write SetWijzer;
    end;

procedure Register;

implementation

function Min(a, b: integer): integer;
Begin
  if a <= b then Result := a else Result := b
End;

procedure TVormKlok.SetWijzer;
Begin
  FWijzers[TWijzerSoort(index)] := Value;
  Invalidate
End;

function TVormKlok.GetWijzer;
Begin
  Result := FWijzers[TWijzerSoort(index)]
End;

procedure TVormKlok.SetGetallen;
Begin
  FGetallen := Value;
  Case Value of
  Geen: CijferSet := [];
  DrieZesNegenTwaalf: CijferSet := [3, 6, 9, 12];
  Alle: CijferSet := [1..12] End;
  Invalidate
End;

function TVormKlok.GetGetallen;
Begin
  Result := FGetallen
End;

constructor TVormKlok.Create;
var W: TWijzerSoort;
Begin
  inherited Create(AOwner);
  FGetallen := Alle; CijferSet := [1..12]; Font.Size := 6;
  Width := 128; height := 128;
  For W := Low(TWijzerSoort) to High(TWijzerSoort) do Begin
      FWijzers[W] := TWijzer.Create;
      FWijzers[W].Dikte := 1 End;
  FWijzers[W_Sec].Kleur := clRed;
  FWijzers[W_Sec].Lengte := 90;
  FWijzers[W_Min].Lengte := 90;
  FWijzers[W_Uur].Lengte := 75;
  FFactorCijferRing := 100;
  FDigital := false;
  Oud_Min := -1;
  Klok := TTimer.Create(nil);
  Klok.OnTimer := NieuweStand
End;

destructor TVormKlok.Destroy;
Begin
  FWijzers[W_Uur].Free; FWijzers[W_Min].Free; FWijzers[W_Sec].Free;
  Klok.Free;
  inherited Destroy
End;

procedure TVormKlok.Paint;
Begin
  inherited paint;
  NieuweStand(nil)
End;

procedure TVormKlok.NieuweStand;
var TijdTekst: ShortString;
    SinGr, CosGr, Factor: double;
    Uur, Minuut, Sec, Kleintje, L, DiktePen, u, R: word;
    KleurPen: TColor;
const Pi30 = Pi / 30; Pi6 = Pi / 6;
      CosUur: array[1..12] of double
            = (0.866, 0.5, 0, -0.5, -0.866, -1, -0.866, -0.5, 0, 0.5, 0.866, 1);
      SinUur: array[1..12] of double
            = (0.5, 0.866, 1, 0.866, 0.5, 0, -0.5, -0.866, -1, -0.866, -0.5, 0);
Begin
  If not Visible then Exit;
  DecodeTime(Now, Uur, Minuut, Sec, Kleintje);
  If Sender <> nil then Begin
     If FSeconde or (Oud_Min <> Minuut) then Invalidate; // Niets tekenen als er niets veranderd is
     Exit End; // Invalidate zorgt ervoor dat andere componenten weten dat zij eventueel opnieuw getekend moeten worden
  Oud_Min := Minuut;
  Fx_c := Width shr 1; Fy_c := Height shr 1; // Default positie van de as
  If Vorm in [DriehoekOmhoog..DriehoekOmlaag] then Factor := 0.0033
  else Factor := 0.005;
  Case Vorm of
  DriehoekOmhoog: Fy_c := round((2 * Height) / 3);
  DriehoekRechts: Fx_c := round(Width / 3);
  DriehoekLinks:  Fx_c := round((2 * Width) / 3);
  DriehoekOmlaag: Fy_c := round(Height / 3)
  End;
  Canvas.Font := Font;
  If Digital then Begin
     If FSeconde then LongTimeFormat := 'hh:mm:ss'
     else LongTimeFormat := 'hh:mm';
     TijdTekst := TimeToStr(Now);
     With Canvas do
          TextOut((Width - TextWidth(TijdTekst)) div 2,
                  (Height - TextHeight(TijdTekst)) div 2, TijdTekst);
     Exit End;
  With Canvas do Begin
       R := Round(FFactorCijferRing * Factor * Min(Width, Height) - TextWidth('12'));
       For u := 1 to 12 do
           If u in CijferSet then
           TextOut(Fx_c + round(R * SinUur[u]) - TextWidth(IntToStr(u)) shr 1
           , Fy_c - round(R * CosUur[u]) - TextHeight(IntToStr(u)) shr 1, IntToStr(u));
       DiktePen := Pen.Width; KleurPen := Pen.Color;
       With FWijzers[W_Uur] do Begin
            If Height > Width then L := Round(Factor * Lengte * Width)
            else L := Round(Factor * Lengte * Height);
            If Uur > 12 then Dec(Uur, 12);
            MoveTo(Fx_c, Fy_c);
            SinGr := sin(Pi6 * (Uur + Minuut / 60));
            CosGr := cos(Pi6 * (Uur + Minuut / 60));
            Pen.Width := Dikte; Pen.Color := Kleur;
            LineTo(Fx_c + round(L * SinGr), Fy_c - round(L * CosGr)) End;
       With FWijzers[W_Min] do Begin
            If Height > Width then L := Round(Factor * Lengte * Width)
            else L := Round(Factor * Lengte * Height);
            MoveTo(Fx_c, Fy_c);
            SinGr := sin(Pi30 * Minuut); CosGr := cos(Pi30 * Minuut);
            Pen.Width := Dikte; Pen.Color := Kleur;
            LineTo(Fx_c + round(L * SinGr), Fy_c - round(L * CosGr)) End;
       If FSeconde then With FWijzers[W_Sec] do If Dikte > 0 then Begin
          If Height > Width then L := Round(Factor * Lengte * Width)
          else L := Round(Factor * Lengte * Height);
          MoveTo(Fx_c, Fy_c);
          SinGr := sin(Pi30 * Sec); CosGr := cos(Pi30 * Sec);
          Pen.Width := Dikte; Pen.Color := Kleur;
          LineTo(Fx_c + round(L * SinGr), Fy_c - round(L * CosGr)) End;
       Pen.Width := DiktePen; Pen.Color := KleurPen
       End
End;

function TVormKlok.GetCijferRing: word;
begin
  Result := FFactorCijferRing
end;

procedure TVormKlok.SetCijferRing(Value: word);
begin
  If FFactorCijferRing = Value then Exit;
  FFactorCijferRing := Value;
  Invalidate
end;

procedure Register;
begin
  RegisterComponents('LevendeVormen', [TVormKlok])
end;

end.
