unit Vormen;

interface

uses
  Windows, Classes, Graphics, Controls;

type
  TVormen = (Rechthoek, Vierkant, RondeRechthoek, RondVierkant, Ellips
            , Cirkel, Honing, HoningRaat, DriehoekOmhoog, DriehoekLinks
            , DriehoekRechts, DriehoekOmlaag);
  TVorm = class(TGraphicControl)
  private
    FVorm: TVormen;
    FPen: TPen;
    FBrush, FAchterBrush: TBrush;
    procedure SetVorm(Value: TVormen);
    procedure SetBrush(Value: TBrush);
    procedure SetAchterBrush(Value: TBrush);
    procedure SetPen(Value: TPen);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Achtergrond: TBrush read FAchterBrush write SetAchterBrush;
    procedure StyleChanged(Sender: TObject);
    property Brush: TBrush  read FBrush write SetBrush;
    property Pen:   TPen    read FPen   write SetPen;
    property Vorm:  TVormen read FVorm  write SetVorm;
    property Height default 62; // Publiceer deze twee properties en
    property Width  default 62; // geef ze een nieuwe default waarde
    property DragCursor;        // Publiceer deze drie properties, de
    property DragMode;          // defaultwaarde is al goed
    property Visible;           //
    property OnClick;           // Publiceer alle muis-events
    property OnDblClick;        // zodat een gebruiker er eventueel
    property OnDragDrop;        // iets mee kan doen!
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    end;

procedure Register;

implementation

constructor TVorm.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FPen := TPen.Create;
  FPen.OnChange := StyleChanged;
  FBrush := TBrush.Create;
  FBrush.OnChange := StyleChanged;
  FAchterBrush := TBrush.Create;
  FAchterBrush.Style := bsClear;
  FAchterBrush.OnChange := StyleChanged;
  Height := 62; Width := 62
end;

destructor TVorm.Destroy;
begin
  FPen.Free;
  FBrush.Free;
  FAchterBrush.Free;
  inherited Destroy
end;

procedure TVorm.SetVorm(Value: TVormen);
Begin
  If Vorm <> Value then Begin
     FVorm := Value;
     Invalidate End
End;

procedure TVorm.StyleChanged;
Begin
  Invalidate
End;

procedure TVorm.SetBrush(Value: TBrush);
Begin
  FBrush.Assign(Value)
End;

procedure TVorm.SetAchterBrush(Value: TBrush);
Begin
  FAchterBrush.Assign(Value)
End;

procedure TVorm.SetPen(Value: TPen);
Begin
  FPen.Assign(Value)
End;

procedure TVorm.Paint;
var Points: array[0..5] of TPoint;
    Marge, Rechts, Boven: integer;
Begin
  If not Visible and not (csDesigning in ComponentState) then Hide else
  With Canvas do Begin
     If Visible then Begin
        Brush := FAchterBrush;
        FillRect(ClientRect);
        Pen := FPen;
        Brush := FBrush End
     else Begin
        Brush.Style := bsClear;
        FillRect(ClientRect);
        Pen.Width := 1;
        Pen.Style := psDash End;
     Marge := Pen.Width shr 1;
     Rechts := Width - Marge; Boven := Height - Marge;
     With Pen do If not Odd(Width) and (Width <> 0) then
        Begin Inc(Rechts); Inc(Boven) End;
     If FVorm >= Honing then // Overal waar Polygon wordt gebruikt:
        Begin Dec(Rechts); Dec(Boven) End;
     Case FVorm of
       Rechthoek: Rectangle(Marge, Marge, Rechts, Boven);
       RondeRechthoek:
           RoundRect(Marge, Marge, Rechts, Boven, Width shr 2, Height shr 2);
       Vierkant:
           If Height > Width then
              Rectangle( Marge, (Height + Marge - Rechts) shr 1
                       , Rechts, (Height - Marge + Rechts) shr 1 )
           else Rectangle( (Width + Marge - Boven) shr 1, Marge
                         , (Width - Marge + Boven) shr 1, Boven );
       RondVierkant:
           If Height > Width then
              RoundRect( Marge, (Height + Marge - Rechts) shr 1
                       , Rechts, (Height - Marge + Rechts) shr 1
                       , Width shr 2, Width shr 2 )
           else RoundRect( (Width + Marge - Boven) shr 1, Marge
                         , (Width - Marge + Boven) shr 1, Boven
                         , Height shr 2, Height shr 2 );
       Ellips: Ellipse(Marge, Marge, Rechts, Boven);
       Cirkel:
           If Height > Width then
              Ellipse( Marge, (Height + Marge - Rechts) shr 1
                     , Rechts, (Height - Marge + Rechts) shr 1)
           else Ellipse( (Width + Marge - Boven) shr 1, Marge
                       , (Width - Marge + Boven) shr 1, Boven);
       Honing, HoningRaat: Begin
           If FVorm = HoningRaat then
              Height := Round(0.8660254037844 * Width);
           Points[0].x := Round(Width/4); Points[0].y := Marge;
           Points[1].x := Width - Points[0].x - 1; Points[1].y := Marge;
           Points[2].x := Rechts; Points[2].y := (Height - 1) shr 1;
           Points[3].x := Points[1].x; Points[3].y := Boven;
           Points[4].x := Points[0].x; Points[4].y := Boven;
           Points[5].x := Marge; Points[5].y := (Height - 1) shr 1;
           Polygon(Points) End;
       DriehoekOmhoog: Begin
           Points[0].x := Width shr 1; Points[0].y := Marge;
           Points[1].x := Rechts;      Points[1].y := Boven;
           Points[2].x := Marge;       Points[2].y := Boven;
           Polygon(Slice(Points, 3)) End;
       DriehoekRechts: Begin
           Points[0].x := Marge;  Points[0].y := Marge;
           Points[1].x := Marge;  Points[1].y := Boven;
           Points[2].x := Rechts; Points[2].y := Height shr 1;
           Polygon(Slice(Points, 3)) End;
       DriehoekLinks: Begin
           Points[0].x := Rechts; Points[0].y := Marge;
           Points[1].x := Rechts; Points[1].y := Boven;
           Points[2].x := Marge;  Points[2].y := Height shr 1;
           Polygon(Slice(Points, 3)) End;
       DriehoekOmlaag: Begin
           Points[0].x := Width shr 1; Points[0].y := Boven;
           Points[1].x := Rechts;      Points[1].y := Marge;
           Points[2].x := Marge;       Points[2].y := Marge;
           Polygon(Slice(Points, 3)) End
       End
     End
End;

procedure Register;
begin
  RegisterComponents('LevendeVormen', [TVorm])
end;

end.
